---
subtitle: Framework serveur et CSS
---

## Le framework Flask

<https://flask.palletsprojects.com/en/2.2.0/>

> Flask is a lightweight WSGI **web application framework**. [...] It began as a simple _wrapper_ around **Werkzeug and Jinja** and has become one of the most popular Python web application frameworks.

. . .

Voir le cours [Programmation Web pour la Bio-Informatique](https://perso.liris.cnrs.fr/pierre-antoine.champin/2019/progweb-python/cours/cm2.html) de Pierre-Antoine Champin qui contient un tutoriel Flask.

---

La principal alternative Python est <https://www.djangoproject.com/>.
Les deux projets Flask et Django font des choix différents :

- Flask est un _micro framework_ : **léger** et **extensible**
- Djnago est un _framework_ "lourd"/"classique" : **complet** et **intégré**

---

### Premier serveur web en Python

```python
from flask import Flask

app = Flask(__name__)

@app.route("/")
def hello_world():
    return "<p>Hello, World!</p>"
```

. . .

```bash
export FLASK_APP=minimal.py     # l'app Flask
export FLASK_ENV=development    # mode dev != production
flask run                       # ouvre le serveur web et lance l'app
```

---

### Routage Flask

- **traiter** une _requête_ HTTP = **construire** de _réponse_ HTTP
- le traitements sont définis par des fonctions (appellés _handlers_)
  - elles prennent en paramètre les valeurs extraites de la requête
  - elles modifient les _en-têtes_ **et** le _corps_ de la réponse _retournée_

. . .

Le framework Flask se charge de tout la plomberie HTTP, le développeur ne programme **que** les traitements au dessus

---

L'association d'une fonction à **une route** (un modèle d'URL) se fait via le **décorateur** `@app.route`

- une syntaxe particulière Python avec `@` pour les décorateur :
  - décorateur = _fonction qui "décore" une fonction_
    - prend **une fonction en paramètre** (à décorer)
    - renvoie **une fonction** (une fois décorée)
  - utilisé dans [me module standard `functools`](https://docs.python.org/3/library/functools.html)
  - par exemple <https://realpython.com/primer-on-python-decorators/>

---

[Jinja](https://jinja.palletsprojects.com/en/3.1.x/) est utilisé pour construire corps de la réponse, c.-à-d. le document renvoyé au navigateur.
Jinja est complètement intégré à Flask.

. . .

![Fichier `users.jinja2.html`](img/users.jinja2.html.png)

. . .

```python
@app.route("/users/")
def users():
    """Tous les utilisateurs"""
    return render_template("users.jinja2.html", users=users_get())
```

Fichier `server.py`

---

- Les routes peuvent contenir _des paramètres_
- Des glogaux et fonctions de Flask permettent de manipuler l'environnement
  - requête, réponse, variables de session, variables d'application
  - par exemple : `url_for`, `abort`, `request` etc.

```python
@app.route("/status/<status_code>")
def status(status_code):
    """route d'erreur pour démo/tests"""
    app.logger.warning(status_code)
    return abort(int(status_code))
```

---

### Démonstration

On montre une [application web Flask simple](exemple_flask/) :

- gestion de _templates_ Jinja
  - utilisation de l'héritage
- formulaire HTML d'ajout d'utilisateur
  - éléments spécifiques `<form>`, `<input>` etc.
- une base de données SQLite3 pour stocker les utilisateurs

## Frameworks CSS

Ensemble **de classes** pour _toute_ la mise en page :

- positionnement et colonnes/grilles
  - e.g., via flexbox voire grid
- éléments HTML
- formulaires et composants
  - composition de plusieurs éléments
  - e.g., menu, modal, pagination, card
- petit outillage :
  - e.g., couleurs, espace, fontes,

La compléxité/richesses des propriétés et valeurs CSS va être abstraite par un système de classes _propre_ au framework

### Méthodologie Block-Element-Model

![[`the_coding_love()`](https://thecodinglove.com/when-im-searching-for-a-meaningful-variable-name)](img/searching_meaningful_variable_name2.png)

<!-- (https://thecodinglove.com/content/028/searching_meaningful_variable_name2.png) -->

---

Extrait de <http://getbem.com/introduction/> :

(voir aussi <https://css-tricks.com/bem-101/>)

- <span style="color:#76b24f">Block</span> : _Standalone_ entity that is meaningful on its own.
  - `header, container, menu, checkbox, input`
- <span style="color:#4f91b2">Element</span>: A _part_ of a block that has _no standalone meaning_ and is semantically _tied_ to its block.
  - `menu-item, checkbox-caption, header-title`
- <span style="color:#d5635d">Modifier</span> : A flag on a block or element. Use them to change appearance or behavior.
  - `disabled, highlighted, checked, fixed, big, red`

---

![Source [Get BEM](http://getbem.com/introduction/)](img/github_captions.jpg)

---

### Sous le capot

![](img/bem-btn-modifier.png)

. . .

![Source [Get BEM](http://getbem.com/introduction/)](img/bem-btn-css-code.png)

---

### Exemple : Bulma.io

![](https://bulma.io/images/bulma-logo.png)

- <https://bulma.io/documentation/>
- <https://bulma.io/expo/>
- <https://github.com/jgthms/bulma/blob/master/docs/_data/expo.json>
- <https://github.com/troymott/bulma-book-code>

Les alternatives sont <https://getbootstrap.com/>, <https://purecss.io/> (avec/sans JS, minimaliste/riche, intégré/extensible)

---

### Approche CSS utilitaire/atomique

Dans l'approche CSS utilitaire/atomique

- on ne va **pas** proposer des classes pour des composants/éléments
- on ne va **pas** non plus écrire ses propres class CSS
- on va mettre an page en utilisant des classes **très simples**, _mono-fonctionnelles_

  - presque comme si les valeurs et propriétés de l'attribut `style` devenaient des **noms de classes**

- <https://tailwindcss.com/docs/utility-first>
- <https://css-tricks.com/lets-define-exactly-atomic-css/>

---

#### TailwindCSS

<https://tailwindcss.com/> est un des principaux représentants.

De plus en plus populaire

::: columns

::: column

##### Custom CSS BEM style

```html
<div class="chat-notification">
  <div class="chat-notification-logo-wrapper">
    <img class="chat-notification-logo" src="/img/logo.svg" alt="ChitChat Logo" />
  </div>
  <div class="chat-notification-content">
    <h4 class="chat-notification-title">ChitChat</h4>
    <p class="chat-notification-message">You have a new message!</p>
  </div>
</div>
```

:::

::: column

##### Tailwindcss

```html
<div class="p-6 max-w-sm mx-auto bg-white rounded-xl shadow-lg flex items-center space-x-4">
  <div class="shrink-0">
    <img class="h-12 w-12" src="/img/logo.svg" alt="ChitChat Logo" />
  </div>
  <div>
    <div class="text-xl font-medium text-black">ChitChat</div>
    <p class="text-slate-500">You have a new message!</p>
  </div>
</div>
```

:::

:::

## Conclusion

?

